<?php

namespace App\Admin\Controllers;

use OpenAdmin\Admin\Controllers\AdminController;
use OpenAdmin\Admin\Form;
use OpenAdmin\Admin\Grid;
use OpenAdmin\Admin\Show;
use \App\Models\Clients;
use \App\Models\SubZones;
use \App\Models\Zones;


class ClientsController extends AdminController
{
    /**
     * Title for current resource.
     *
     * @var string
     */
    protected $title = 'Clients';

    /**
     * Make a grid builder.
     *
     * @return Grid
     */
    protected function grid()
    {
        $grid = new Grid(new Clients());

        $grid->column('id', __('Id'));
        $grid->column('first_name', __('First Name'));
        $grid->column('last_name', __('Last Name'));
        $grid->column('phone', __('Phone'));
        $grid->column('email', __('Email'));
        $grid->column('id_number', __('ID Number'));
        $grid->column('zones.name', __('Zone'));
        $grid->column('szone.name', __('Sub Zone'));
        // $grid->column('contract_date', __('Contract date'));
        // $grid->column('contract_expiry', __('Contract expiry'));
        // $grid->column('photo', __('Photo'));
        // $grid->column('address', __('Address'));
        // $grid->column('status', __('Status'));
        // $grid->column('county', __('County'));
        // $grid->column('sub_county', __('Sub county'));
        // $grid->column('location', __('Location'));
        $grid->column('created_at', __('Created at'));//->format('Y-m-d');
        $grid->column('updated_at', __('Updated at'));

        return $grid;
    }

    /**
     * Make a show builder.
     *
     * @param mixed $id
     * @return Show
     */
    protected function detail($id)
    {
        $show = new Show(Clients::findOrFail($id));

        $show->field('id', __('Id'));
        $show->field('first_name', __('First name'));
        $show->field('last_name', __('Last name'));
        $show->field('phone', __('Phone'));
        $show->field('email', __('Email'));
        $show->field('id_number', __('Id number'));
        $show->field('zone_id', __('Zone id'));
        $show->field('sub_zone', __('Sub zone'));
        $show->field('contract_date', __('Contract date'));
        $show->field('contract_expiry', __('Contract expiry'));
        $show->field('photo', __('Photo'));
        $show->field('address', __('Address'));
        $show->field('status', __('Status'));
        $show->field('county', __('County'));
        $show->field('sub_county', __('Sub county'));
        $show->field('location', __('Location'));
        $show->field('created_at', __('Created at'));
        $show->field('updated_at', __('Updated at'));

        return $show;
    }

    /**
     * Make a form builder.
     *
     * @return Form
     */
    protected function form()
    {
        $form = new Form(new Clients());
        $form->tab('Basic info', function ($form) {
        $form->text('first_name', __('First Name'))->required();
        $form->text('last_name', __('Last Name'))->required();
        $form->select('gender','Gender')->options([1 => 'Male', 2 => 'Female', 3 => 'Other']);
        $form->phonenumber('phone', __('Phone Number'))->options(['mask' => '9999 999 999'])->required()->placeholder('Phone Number');
        $form->text('id_number', __('ID Number'))->required()->placeholder('ID Number');
        $form->email('email', __('Email Address'));
        $form->select('zone_id', __("Zone"))->options(Zones::all()->pluck('name', 'id'))->load('zone_id', url('api/sub-zones'))->required();
        $form->select('sub_zone', __("Sub Zone"))->options(SubZones::all()->pluck('name', 'id'))->required();
        $form->date('contract_date', __('Contract Date'))->required();
        $form->date('contract_expiry', __('Contract Expiry'));
         })->tab('More information', function ($form) {
        $form->image('photo', __('Photo'))->thumbnail([
            //'small' => [100, 100],
            'small' => [200, 200],
            //'small' => [300, 300],
        ]);
        $form->textarea('address', __('Address'));
        $form->text('status', __('Status'))->default('1');
        $form->text('county', __('County'));
        $form->text('sub_county', __('Sub county'));
        $form->text('location', __('Location'));
        });

        return $form;
    }
}